/*
  Remote SDR
  Interface Serial-USB 
  - Frequency Tuning
  - TX/RX state change
  - CW Key
  F1ATB 26 December 2021
  20. feb. 2024 LA3FY:
  Handling the rotary encoder totally rewritten
  Before compiling, please install the RotaryEncoder with the Library manager!
  Search for "RotaryEncoder" by Matthias Hertel
  29. feb. 2024 LA3FY
  New function added. PTT button can now be uses as a toggle switch (as earlier) or act as
  a microphone PTT button. Tx will be on as long as PTT is pushed. To change between functions,
  the PTT button must be pushed and kept pushed during power on. Just keep the button pressed
  for 5-10sec while connecting power. Mode will be stored in EEPROM and will be used until
  next time PTT button is pushed when power is connected. Then it's very easy to 
  change between the two PPT modes.
*/
#include <Arduino.h>
#include <RotaryEncoder.h>
#include <EEPROM.h>

#define PIN_IN1 2
#define PIN_IN2 3

// digital pin 4 has a pusbutton attached to it
int pushButton = 4;
int OldButton = 1;

//CW Key
int LeftKey = 5;
int RightKey = 6;
int OldLeft = 1;
int OldRight = 1;

boolean PTTtoggle = true;
boolean PTTon = false;

int Delay = 1; //this value can be increased to make the encoder less sensitive
               //a value of 10 makes my 100 pulses/revolution encoder almost perfect


// A pointer to the dynamic created rotary encoder instance.
// This will be done in setup()
RotaryEncoder *encoder = nullptr;

void checkPosition()
{
  encoder->tick(); // just call tick() to check the state.
}

void setup()
{
  Serial.begin(115200);
  // make the pushbutton's pin an input:
  pinMode(pushButton, INPUT_PULLUP);
  pinMode(LeftKey, INPUT_PULLUP);
  pinMode(RightKey, INPUT_PULLUP);
  int eeAddress = 0;   //Location where we want to store PTT to act as toggle switch or not
  PTTtoggle = EEPROM.read(eeAddress);
  if ((digitalRead(pushButton)==LOW)) {
    Delay=20;
    PTTtoggle = !PTTtoggle;
    EEPROM.update(eeAddress,PTTtoggle);
    while (digitalRead(pushButton)==LOW) {}
  }
  // setup the rotary encoder functionality

  // use FOUR3 mode when PIN_IN1, PIN_IN2 signals are always HIGH in latch position.
  // encoder = new RotaryEncoder(PIN_IN1, PIN_IN2, RotaryEncoder::LatchMode::FOUR3);

  // use FOUR0 mode when PIN_IN1, PIN_IN2 signals are always LOW in latch position.
   encoder = new RotaryEncoder(PIN_IN1, PIN_IN2, RotaryEncoder::LatchMode::FOUR0);

  // use TWO03 mode when PIN_IN1, PIN_IN2 signals are both LOW or HIGH in latch position.
  //encoder = new RotaryEncoder(PIN_IN1, PIN_IN2, RotaryEncoder::LatchMode::TWO03);

  // register interrupt routine
  attachInterrupt(digitalPinToInterrupt(PIN_IN1), checkPosition, CHANGE);
  attachInterrupt(digitalPinToInterrupt(PIN_IN2), checkPosition, CHANGE);
} // setup()


void loop()
{
  static int pos = 0;
  static char direction='U';

  Delay = 20;
  if (PTTtoggle==true) {
  // Read PushButton and B0 will be sent when button pushed and B1 when releasing button
    int Button = digitalRead(pushButton);
    if (OldButton != Button) {
        Serial.println("B"+String(Button));
        OldButton = Button;
        Delay = 10;
    }
    
  }
  else {
    //The B0 B1 sequence will be sendt when PTT is pressed - Tx will be activated
    if ((digitalRead(pushButton)==LOW) && (!PTTon)) {
      PTTon=true;
      int Button = digitalRead(pushButton);
      if (OldButton != Button) {
        Serial.println("B"+String(Button));
        Delay = 20;
        Serial.println("B"+String(!Button));
        OldButton = Button;
        Delay = 20;
      }
    }
    //The B0 B1 sequence will be sent when PPT is released - Tx will be deactivated
    if ((digitalRead(pushButton)==HIGH) && (PTTon)) {
      PTTon=false;
      int Button = digitalRead(pushButton);
      if (OldButton != Button) {
        Serial.println("B"+String(!Button));
        Delay = 20;
        Serial.println("B"+String(Button));
        OldButton = Button;
        Delay = 20;
      }
    }
    
  }



    // Read LeftKey
    int kLeft = digitalRead(LeftKey);
    if (OldLeft != kLeft) {
        Serial.println("L"+String(kLeft));
        OldLeft = kLeft;
        Delay = 2;
    }
    // Read RighttKey
    int kRight = digitalRead(RightKey);
    if (OldRight != kRight) {
        Serial.println("R"+String(kRight));
        OldRight = kRight;
        Delay = 2;
    }

  encoder->tick(); // just call tick() to check the state.

  int newPos = encoder->getPosition();
  if (pos != newPos) {
    if (((int)(encoder->getDirection()))<0) {direction='D';}
    else
    {direction='U';}
    Serial.print(direction);
    Serial.println(newPos-pos);
    pos = newPos;
  } // if
  delay(Delay);        // delay in between reads for stability
} // loop ()



